@@ -5,36 +5,26 @@
 #include "core/exceptions/OperationNotSupportedException.hpp"
 #include "core/exceptions/NullPtrException.hpp"
 
-TEST(net_operations_test, graph_union)
+TEST(operations_test, graph_union)
 {
 
     // creating some simple graphs
 
-    auto v1 = std::make_shared<const uu::net::Vertex>("v1");
-    auto v2 = std::make_shared<const uu::net::Vertex>("v2");
-    auto v3 = std::make_shared<const uu::net::Vertex>("v3");
+    auto g1 = std::make_unique<uu::net::Network>("g1", uu::net::EdgeDir::DIRECTED);
+    auto g2 = std::make_unique<uu::net::Network>("g2", uu::net::EdgeDir::DIRECTED);
 
-    auto dir = uu::net::EdgeDir::DIRECTED;
-    auto e1 = std::make_shared<const uu::net::Edge>(v1.get(), v2.get(), dir);
-    auto e2 = std::make_shared<const uu::net::Edge>(v2.get(), v3.get(), dir);
-    auto e3 = std::make_shared<const uu::net::Edge>(v3.get(), v2.get(), dir);
-    auto e4 = std::make_shared<const uu::net::Edge>(v3.get(), v2.get(), dir);
-
-    auto g1 = std::make_unique<uu::net::Network>("g1", dir);
-    auto g2 = std::make_unique<uu::net::Network>("g2", dir);
-
-    g1->vertices()->add(v1);
-    g1->vertices()->add(v2);
-    g1->vertices()->add(v3);
+    auto v1 = g1->vertices()->add("v1");
+    auto v2 = g1->vertices()->add("v2");
+    auto v3 = g1->vertices()->add("v3");
 
     g2->vertices()->add(v2);
     g2->vertices()->add(v3);
 
-    g1->edges()->add(e1);
-    g1->edges()->add(e2);
+    g1->edges()->add(v1, v2);
+    g1->edges()->add(v2, v3);
 
-    g2->edges()->add(e2);
-    g2->edges()->add(e3);
+    g2->edges()->add(v2, v3);
+    g2->edges()->add(v3, v2);
 
     // union
 
@@ -47,14 +37,15 @@ TEST(net_operations_test, graph_union)
 
     // Exception handling
 
-    // multiple edges with same end-vertices
+    /* multiple edges with same end-vertices
 
-    g1->edges()->add(e4);
+    g1->edges()->add(v3, v2);
 
     EXPECT_THROW(
         uu::net::graph_union(g1.get(), g2.get()),
         uu::core::OperationNotSupportedException
     );
+     */
 
     // null parameters
 
