@@ -56,6 +56,7 @@ try:
     # creation
     empty,
     grow, evolution_pa, evolution_er,
+    generate_communities,
     # io
     read, write,
     # info
@@ -78,8 +79,8 @@ try:
     layer_summary, layer_comparison,
     distance,
     # clustering
-    clique_percolation, glouvain, abacus, infomap,
-    modularity,
+    clique_percolation, glouvain, abacus, infomap, flat_ec, flat_nw, mdlp,
+    modularity, nmi, omega_index,
     # visualization
     layout_multiforce, layout_circular,
     # networkx
